/*********************************************************************
  Routines for WAV-files
  support for 8, 12 and 16 bit samples (12 bit are treated as 16 bit)

  part of: Hide4PGP by Heinz Repp
*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "hide4pgp.h"

static long filelength, position;


/*********************************************************************
  skip_to:   searches from current position for specified subchunk
  parameter: pointer to string with chunk type
  returns:   length, 0L if not found
             file read pointer points behind the chunk length entry
*********************************************************************/
long skipto (char *subtype)
{ long length;
  int found;

  while (position < filelength)
  { if (fseek (StegFile, position, SEEK_SET)) break;
    found = next_is (subtype);
    fread (&length, sizeof (long), 1, StegFile);
    position += length + 9 & ~1L;  /* word aligned */
    if (found)
      return length;
  } /* endwhile */
  return 0L;
} /* skipto subchunk */


/*********************************************************************
  wav_prepare: check WAV file characteristics, optionally output
             specification data
  parameter: none
  returns:   data width flag, TRUE for 16 (or 12) bit, FALSE for 8
*********************************************************************/
int wav_prepare (void)
{ long sample_fq;
  int bit_p_spl;

  if (! next_is ("IFF") ||
      (fread (&filelength, sizeof (long), 1, StegFile),
       ! next_is ("WAVE")))
  { fputs ("Error (WAV): File identification failed.\n", stderr);
    exit (20);
  } else if (verbose)
    fputs ("Microsoft RIFF / subtype WAVE format detected.\n", stderr);
  filelength += 8;
  position = 12;
  if (skipto ("fmt ") == 0L) unexp_EOF ();
  if (GETWORD (StegFile) != 1)
  { fputs ("Error (WAV): Sound data not in PCM format.\n", stderr);
    exit (21);
  } /* endif format */
  if (verbose > 1)
    fprintf (stderr, "No. of channels  : %i\n", GETWORD (StegFile));
  else GETWORD (StegFile);
  fread (&sample_fq, sizeof (long), 1, StegFile);
  if (verbose > 1)
    fprintf (stderr, "Sample frequency : %li Hz.\n", sample_fq);
  fseek (StegFile, 6, SEEK_CUR);
  if ((bit_p_spl = GETWORD (StegFile)) == 12 && hiding)
  { fseek (StegFile, -2L, SEEK_CUR);
    PUTWORD (bit_p_spl = 16, StegFile);
    if (verbose)
      fputs ("Sample was 12 bit - changed to -> ", stderr);
  } /* endif 12 bit when hiding */
  if (verbose)
    fprintf (stderr, "Sample resolution: %i bit.\n", bit_p_spl);
  switch (bit_p_spl)
  { case 8:
      return FALSE;
    case 16:
      return TRUE;
    default:
      fputs ("Error (WAV): This resolution is not supported.\n", stderr);
      exit (22);
  } /* switch bit_p_spl */
} /* wav_prepare */


/*********************************************************************
  wav_nextblk: advance read pointer to next data block
  parameter: none
  returns:   length of continous data block, 0L if none
*********************************************************************/
long wav_nextblk (void)
{ return skipto ("data");
} /* wav_nextblk */
